import sql from '../../sql.mjs';
import lib from '../../lib.mjs';

export default router => {
  router.group(/^\/api\/v2\/admin\/(?<postid>\d+)\/tags/, group => {
    group.get(/$/, lib.auth, async (req, res) => {
      // get tags
      if(!req.params.postid) {
        return res.json({
          success: false,
          msg: 'missing postid'
        });
      }

      return res.json({
        success: true,
        tags: await lib.getTags(+req.params.postid)
      });
    });

    group.post(/$/, lib.auth, async (req, res) => {
      // assign and/or create tag
      if(!req.params.postid || !req.post.tagname) {
        return res.json({
          success: false,
          msg: 'missing postid or tag'
        });
      }

      const postid = +req.params.postid;
      const tagname = req.post.tagname?.trim();
  
      if(tagname.length >= 45) {
        return res.json({
          success: false,
          msg: 'tag is too long!'
        });
      }
  
      try {
        let tagid = (
          await sql("tags")
            .select("id")
            .whereRaw("normalized = slugify(?)", [ tagname ])
          )?.[0]?.id;

        if(!tagid) { // create new tag
          tagid = (await sql("tags").returning("id").insert({
            tag: tagname
          }))[0];
        }
        await sql("tags_assign").insert({
          tag_id: tagid,
          item_id: postid,
          user_id: req.session.id
        });
      } catch(err) {
        return res.json({
          success: false,
          msg: err.message,
          tags: await lib.getTags(postid)
        });
      }

      return res.json({
        success: true,
        postid: postid,
        tag: tagname,
        tags: await lib.getTags(postid)
      });
    });

    group.put(/\/toggle$/, lib.auth, async (req, res) => {
      // xD
      if(!req.params.postid) {
        return res.json({
          success: false,
          msg: 'missing postid'
        });
      }

      const postid = +req.params.postid;

      if(!(await lib.getTags(postid)).filter(tag => [1,2].includes(tag.id)).length) {
        // insert
        await sql('tags_assign').insert({
          item_id: postid,
          tag_id: 1,
          user_id: req.session.id
        });
      }
      else {
        // update
        await sql('tags_assign')
          .update({
            tag_id: sql.raw('(array[2,1])[tag_id]')
          })
          .whereRaw('tag_id = any(array[1,2])')
          .andWhere('item_id', postid);
      }

      return res.json({
        success: true,
        tags: await lib.getTags(postid)
      });
    });

    group.delete(/\/(?<tagname>.*)/, lib.auth, async (req, res) => {
      // delete tag
      if(!req.params.postid || !req.params.tagname) {
        return res.json({
          success: false,
          msg: 'missing postid or tagname'
        });
      }

      const postid = +req.params.postid;
      const tagname = decodeURIComponent(req.params.tagname);

      const tags = await lib.getTags(postid);

      const tagid = tags.filter(t => t.tag === tagname)[0]?.id ?? null;
      
      if(!tagid) {
        return res.json({
          success: false,
          msg: 'tag is not assigned',
          tags: await lib.getTags(postid)
        });
      }

      let q = sql("tags_assign").where("tag_id", tagid).andWhere("item_id", postid).del();
      if(req.session.level < 50)
        q = q.andWhere("user_id", req.session.id);
      const reply = !!(await q);

      return res.json({
        success: reply,
        tagid,
        tags: await lib.getTags(postid)
      })
    });

  });

  return router;
};